%option nounput
%option noyywrap
%{
  /* extract_examples.lex          Franck BUTELLE 2005 */ 
  #include <stdio.h>
  #include <string.h>
  #include <strings.h>
  #include <errno.h>

  #define CMDZONE 	1
  #define RESULTZONE 	2
  #define DESCRIPTIONZONE 	3
  #define NOTESZONE 	4

  #define MAXSTRING 500	

  int zone;       // current zone : either cmd or result;
  char currentMode[5]="DESCR";   // just to fill enough caracters 
  int debug=0;
  char abrv[100],remind[100]; // abbreviated name, reminder

  FILE *cmdFile,*resultFile;
    /* useful functions prototypes */
  char *trimLeft(char *);
  char *trim(char *);
  char *strtoup(char *);
  char *TeXize(char *);
  char *nopipe(char *);
  void abbrev(char*,char *, char *);

%}
SPACES	[ \t\n\r]*
MODE	("DP"|"REP"|"SFN")
%x CMD_ZONE RESULT_ZONE DESCRIPTION_ZONE NOTES_ZONE OTHER_ZONE FORMAT_ZONE
%%
<INITIAL>^{SPACES}[A-Za-z_]*{SPACES}\n	{
			      yytext[strlen(yytext)-1]='\0'; // get rid of \n
			      abbrev(trim(yytext),abrv,remind);
			      if (debug)
				      fprintf(stderr,"abrv=%s, remind=%s\n",abrv,remind);
  printf ("\\markright{{\\bf %s}%s}",abrv,remind);
  printf ("\\index{%s%s@{\\bf %s}%s}",abrv,remind,abrv,remind);
  //printf ("\\halign to\\hsize{#\\hfil&#\\hfil&\\vtop{\\parindent=0pt\\hsize=8cm\\hangindent.5em\\strut#\\strut}\\cr\n");
  printf ("\\begin{longtable}{l@{\\extracolsep{\\fill}}p{0.83\\textwidth}}\n");
  printf("\\multicolumn{2}{l}{{\\bf %s}%s}\\cr",abrv,remind);
  printf("\\hline\\endfirsthead\n");
  printf("\\multicolumn{2}{l}{{\\bf %s}%s (continued)}\\cr",abrv,remind);
  printf("\\endhead\n");

			       BEGIN(OTHER_ZONE);
			      }
<OTHER_ZONE>^{SPACES}"Format"s?":-".*\n 		{
			      char *temp=index(yytext,'-');
			      temp++;
			      printf ("\\F&%s\\cr\n",TeXize(trim(temp)));
				      BEGIN(FORMAT_ZONE);
			      }
<OTHER_ZONE,FORMAT_ZONE>^{SPACES}"Mode"s?":-".*\n		{
			      char *temp=index(yytext,'-');
			      temp++;
			      printf ("\\M&%s\\cr\n",trim(temp));
			      BEGIN(OTHER_ZONE);
			      }
<OTHER_ZONE>^{SPACES}"Description:-"		{
				      zone=DESCRIPTIONZONE;
				      printf ("\\D&");
				      if (debug)
					fprintf(stderr,"****Definition start");
				      BEGIN(DESCRIPTION_ZONE);
			      }
<DESCRIPTION_ZONE>^{SPACES}"Example"s?":-"{SPACES}{MODE}>$	{
			      char *temp;
				      //printf ("\\cr\n");  // end of definition
			      if (debug)
				fprintf(stderr,"****Example(s) found\n");
			      if (strcmp(currentMode,"DESCR")==0) 
				      memcpy(currentMode,"DP",3);//default schur mode is DPrep
			      zone=CMDZONE;BEGIN(CMD_ZONE);
			      temp=strdup(yytext);
			      temp[yyleng-1]='\0';	//to get rid of >
			      temp=trimLeft(strstr(temp,":-")+2);
			      printf("\\E&{\\tt %s>}\\cr",temp);
			      }
<*>^[:space:]*\n				 // eat empty lines
<CMD_ZONE,RESULT_ZONE,OTHER_ZONE>^{SPACES}NOTES:		{
				      zone=NOTESZONE;
				      BEGIN(NOTES_ZONE);
				      printf("\\N");
			      }
<CMD_ZONE,RESULT_ZONE,INITIAL>^{SPACES}"->".*\n             {
			      char *index;
			      index= strstr(yytext,"->");
			      if (debug)
				fprintf(stderr,"****Command=%s\n",index+2);
			      //index[strlen(index)-1]='\0';//drop the final \n
			      //if (zone==RESULTZONE)
			       // 	printf("\\cr\n"); // end of result
			      printf("\\R&{\\tt %s}\\cr\n",TeXize(trim(index+2)));  // command
				      //Suppose that commands are not longer that one
				      //line
			      if (strcasecmp(index+2,"REP")==0
				 || strcasecmp(index+2,"SFN")==0 
				 || strcasecmp(index+2,"DP")==0) //change mode
				  strcpy(currentMode,strtoup(index+2));
			      zone=RESULTZONE;
			      BEGIN(RESULT_ZONE);
			      }
<FORMAT_ZONE>^.*\n		{		// Format on more than one line for example
				      printf("&%s\\cr\n",TeXize(trim(yytext)));
			      }
<DESCRIPTION_ZONE>^.*\n	  	{
				      printf("&{\\tt %s}\\cr\n",TeXize(trim(yytext)));
			      }
<DESCRIPTION_ZONE>.*\n	  	{
				      printf("{\\tt %s}\\cr\n",TeXize(trim(yytext)));
			      }
<NOTES_ZONE>.*\n	  	{
				      printf("&{\\tt %s}\\cr\n",TeXize(yytext));
			      }
<CMD_ZONE,RESULT_ZONE>^.*\n  	{
			      char temp[100];
			      if (zone==CMDZONE) {
				      fprintf(stderr,"ERROR Command that not start with -> ?:%s\n",yytext);
				      exit(1);
			      }
			      else if (zone==RESULTZONE) {
				      strcpy(temp,trim(yytext));
				      if (*temp!='\0')
					printf("\\multicolumn{2}{l}{\\tt %s}\\cr\n",TeXize(yytext));
			      }
			      else
				      fprintf(stderr,"Error: very strange!\n");
			      }
<DESCRIPTION_ZONE><<EOF>>	{// in case of no example and no notes
				      printf("\\end{longtable}");
				      return(0);
			      }
<NOTES_ZONE><<EOF>> 		{
				      printf("\\end{longtable}");
				      return(0);
			      }
<RESULT_ZONE><<EOF>>		{
				      printf("\\end{longtable}");
				      return(0);
			      }
<CMD_ZONE><<EOF>>		{
				      fprintf(stderr,"strange exit in CMD_ZONE, zone=%d\n",zone);
				      exit(1);
			      }
<<EOF>>  			{
				      fprintf(stderr,"%s%s:strange exit\n",abrv,remind);
				      exit(1);
			      }	
%%
/******************* Some useful functions ******************************/

void
abbrev(char *name, char *abrev, char *reminder)
{
  char *c;
  static char temp[100],temp_r[100];
  
  strcpy(temp,name);
  for (c=temp; *c!='\0' && ((*c >='A' && *c <= 'Z') || (*c == '_')) ; c++);
  if (*c!='\0') {
  	strcpy(temp_r,c);
  	*c='\0';
        strcpy(reminder,TeXize(temp_r));
  } else {
    *temp_r='\0';
  }
  strcpy(abrev,TeXize(temp));
}

char *
trimLeft(char *str) {
    char *c;

      for (c=str ; *c!='\0' && (*c == ' ' || *c =='\n' || *c == '\r' || *c == '\t') ; c++);
        return c;
}

char *
trim (char *str)
{
  char *start, *end;

  start=trimLeft(str);
  if (*start != '\0')
    {
      end = start + strlen (start);
      for (--end; *end == ' ' ||  *end == '\n' || *end == '\r' || *end == '\t';
	   end--);
      *++end = '\0';
    }

  return start;
}

char *
nopipe(char *str)
{
  char *c,*ptr;
  static char res[MAXSTRING];

  for (c=str,ptr=res; *c!='\0' ; c++)
    if (*c=='|') {
       sprintf(ptr,"|\\pipe|");ptr += strlen("|\\pipe|");
    }
    else 
       *ptr++ = *c;
  *ptr='\0';
  return(res);
}

char *
TeXize(char *str) 
{
  char *c,*ptr; 
  static char res[MAXSTRING];

  for (c=str,ptr=res; *c!='\0' ; c++)
  {
    switch (*c)
    { 
    case '{':
        sprintf(ptr,"\\{");ptr += strlen("\\{");
      	break;
    case '}':
        sprintf(ptr,"\\}");ptr += strlen("\\}");
	break;
    case '&':
	sprintf(ptr,"\\&");ptr += strlen("\\&");
	break;
    case '^':
	sprintf(ptr,"\\myhat{}"); ptr += strlen("\\myhat{}");
	break;
    case '~':
	sprintf(ptr,"\\~{}");ptr += strlen("\\~{}");
	break;
    case '#':
	sprintf(ptr,"\\#");ptr += strlen("\\#"); // \n is defined to be negativethinspace
	break;
    case '_':
	*ptr++ ='\\'; *ptr++ = '_';
	break;
    case '\\':
	sprintf(ptr,"\\bs ");ptr += strlen("\\bs ");
	break;
    /*case '>':
	sprintf(ptr,"$>$");ptr += strlen("$>$");
	break;
    case '<':
	sprintf(ptr,"$<$");ptr += strlen("$<$");
	break;*/
/*    case '|':
    	sprintf(ptr,":");ptr += strlen(":");
	break;*/
    case ' ':
        sprintf(ptr,"\\ "); ptr += strlen("\\ ");
    default:
	*ptr++ = *c;
    }
  }
  *ptr='\0';
     return(res); 
}

char *
strtoup(char *str) {
  char *c;

  for (c=str;*c!='\0';c++)
    if (*c>='a' && *c <='z')
      *c -= 'a' -'A';
  return str;
}

int main(int argc,char *argv[]) {
  int i;

    if (argc<2) {
 	fprintf(stderr,"synopsis: %s [-d] helpfile",argv[0]);
	fprintf(stderr,"\nAdd -d option to debug\n");
     	exit(1);
    }
    i=1;
    if (strcmp(argv[1],"-d")==0) {
      debug=1;
      i++;
    }
    if ((yyin=fopen(argv[i],"r"))==NULL) {
      fprintf(stderr,"can't read %s (%s)\n", argv[1], strerror(errno));
      exit(1);
    }

    yylex();
    printf("\n\n");
    return(0);
}
